import { observer } from 'mobx-react';
import { message } from 'antd';
import { Mix, getCanvasPattern, } from '@ant-design/plots';
import { merge, isEmpty, cloneDeep } from '../utils/commons';
import { dataFieldAlias } from '../libs/ht';

const COLOR_SETS = [
  "#FF6B3B",
  "#9FB40F",
  "#76523B",
  "#DAD5B5",
  "#E19348",
  "#F383A2",
];
const COLOR_SETS2 = [
  "#5B8FF9",
  "#61DDAA",
  "#65789B",
];

/**
 * 订单数, 团数: 柱形图
 * 成交率: 折线图
 */
export default observer((props) => {
  const { dataSource, summaryData: areaData, ...config } = props;
  const { xField, yFields, colFields, lineFields, seriesField, tooltip, ...extConfig } = config;
  const diffData0 = dataSource.reduce((r, row) => {
    r.push({ ...row, yField: row[colFields[0]], yGroup: dataFieldAlias[colFields[0]].alias });
    r.push({ ...row, yField: row[colFields[1]], yGroup: dataFieldAlias[colFields[1]].alias });
    return r;
  }, []);
  const diffData1 = dataSource.reduce((r, row) => {
    r.push({ ...row, yField: row[lineFields[1]], yGroup: dataFieldAlias[lineFields[1]].alias });
    return r;
  }, []);
  const calcAxis = isEmpty(diffData0) ? 300 : (Math.max(...diffData0.map(ele => ele.yField))) * 3;
  // const calcAxisC = isEmpty(diffData0) ? 300 : (Math.max(...diffDataPercent.map(ele => ele.yField))) * 3;
  const diffLine = [
    // {
    //   type: 'text',
    //   position: ['start', 0],
    //   content: `同比, 环比`,
    //   offsetX: -15,
    //   style: {
    //     fill: COLOR_SETS[0],
    //     textBaseline: 'bottom',
    //   },
    // },
    // {
    //   type: 'line',
    //   start: [-10, 0],
    //   end: ['max', 0],
    //   style: {
    //     stroke: COLOR_SETS[0],
    //     // lineDash: [2, 2],
    //     lineWidth: 0.5,
    //   },
    // },
  ];

  const pattern = (datum, color) => {
  return getCanvasPattern({
    type: String(datum.yGroup).includes(' ') ? 'line' : '',
    cfg: {
      backgroundColor: color,
    },
  });
  };

  const MixConfig = {
    appendPadding: 15,
    height: 400,
    syncViewPadding: true,
    tooltip: {
      shared: true,
      // customItems: (originalItems) => {
      //   // process originalItems,
      //   const items = originalItems.map((ele) => ({ ...ele, name: ele.data?.extraLine ? ele.name : `${ele.name} ${dataFieldAlias[yField]?.alias || yField}` }));
      //   return items;
      // },
    },
    legend: {
      position: 'top',
      layout: 'horizontal',
      custom: true,
      items: [
        ...['团数', '订单数'].map((ele, ei) => ({
          name: `${ele}`,
          value: `${ele}`,
          marker: {
            symbol: 'square',
            style: {
              fill: COLOR_SETS2[ei],
              r: 5,
            },
          },
        })),
        ...['', '成行率'].map((ele, ei) => ({ // '业绩',
          name: `${ele}`,
          value: `${ele}`,
          marker: {
            symbol: 'hyphen',
            style: {
              stroke: COLOR_SETS[ei],
              r: 5,
              lineWidth: 2
            },
          },
        })),
      ],
    },
    // event: (chart, e) => {
    //   console.log('mix', chart, e);
    //   if (e.type === 'click') {
    //     props.itemClick(e);
    //   }
    // },
    onReady: (plot) => {
      plot.on('plot:click', (...args) => {
        // message.info('请在柱状图上点击, 显示详情');
      });
      plot.on('element:click', (e) => {
        const {
          data: { data },
        } = e;
        // console.log('plot element', data);
        props.itemClick(data);
      });
      // axis-label 添加点击事件
      plot.on('axis-label:click', (e, ...args) => {
        const { text } = e.target.attrs;
        // console.log(text);
        props.itemClick({ [xField]: text });
      });
    },
    plots: [
      {
        type: 'column',
        options: {
          data: diffData0,
          isGroup: true,
          xField,
          yField: 'yField',
          seriesField: 'yGroup',
          // xAxis: false,
          meta: merge({
            ...cloneDeep(dataFieldAlias),
          }),
          // color: '#b32b19',
          // color: '#f58269',
          legend: false, // {},
          // smooth: true,
          yAxis: {
            type: 'linear',
            tickCount: 4,
            min: 0,
            max: calcAxis,
            title: { text: '团数', autoRotate: false, position: 'end' },
          },
          xAxis: {
            label: {
              autoHide: false,
              autoRotate: true,
            },
          },
          label: false,
          color: COLOR_SETS2,
          pattern,
        },
      },
      {
        type: 'line',
        options: {
          data: diffData1,
          isGroup: true,
          xField,
          yField: 'yField',
          seriesField: 'yGroup',
          xAxis: false,
          legend: false, // {},
          meta: merge(
            {
              ...cloneDeep(dataFieldAlias),
            },
            { yField: dataFieldAlias[lineFields[1]] }
          ),
          // color: '#1AAF8B',
          color: COLOR_SETS[1],
          // smooth: true,
          point: {
            size: 4,
            shape: 'cicle',
          },
          yAxis: {
            type: 'linear',
            // tickCount: 4,
            min: 0,
            position: 'right',
            line: null,
            grid: null,
            title: { text: dataFieldAlias[lineFields[1]].label, autoRotate: false, position: 'end' },
          },
          label: {
            style: {
              fontWeight: 700,
              stroke: '#fff',
              lineWidth: 1,
            },
          },
          lineStyle: (datum) => {
            if (String(datum.yGroup).includes(' ')) {
              return {
                lineDash: [4, 4],
                opacity: 0.75,
              };
            }
            return {
              opacity: 1,
            };
          },
        },
      },
      // {
      //   type: 'column',
      //   options: {
      //     data: diffData2,
      //     xField,
      //     yField: 'yField',
      //     seriesField: 'yGroup',
      //     columnWidthRatio: 0.28,
      //     meta: {
      //       // yField: {
      //       //   formatter: (v) => `${v}%`,
      //       // },
      //     },
      //     isGroup: true,
      //     xAxis: false,
      //     yAxis: {
      //       line: null,
      //       grid: null,
      //       label: false,
      //       position: 'left',
      //       // min: -calcAxisC,
      //       // max: calcAxisC/4,
      //       min: -3000,
      //       max: 250,
      //       tickCount: 4,
      //     },
      //     legend: false, // {},
      //     color: COLOR_SETS,
      //     // annotations: diffLine,

      //     minColumnWidth: 5,
      //     maxColumnWidth: 5,
      //     // 分组柱状图 组内柱子间的间距 (像素级别)
      //     dodgePadding: 1,
      //     // 分组柱状图 组间的间距 (像素级别)
      //     // intervalPadding: 20,
      //   },
      // },
    ],
  };
  return <Mix {...MixConfig} />;
});
