import { useContext } from 'react';
import { observer } from 'mobx-react';
import { stores_Context } from './../../config';
import { Typography, Row, Col, Tabs, } from 'antd';
import SearchForm from './../search/SearchForm';
import { KPIObjects } from './../../libs/ht';
import { merge, pick } from './../../utils/commons';
import ProfitTable from './SubjectTable/Profit';
import Count from './SubjectTable/Count';
import Rates from './SubjectTable/Rates';

const Todo = (props) => {
  return <h2>TODO</h2>;
};

const subjectComponents = {
  'sum_profit': ProfitTable,
  'in_order_count': Count,
  'confirm_order_count': Count,
  'depart_order_count': Count,
  'confirm_rates': Rates,
  'praise_rates': Rates,
  'sum_person_num': Count,
};

export const KPIObjectsMapped = KPIObjects.reduce((a, c) => ({ ...a, [String(c.key)]: c }), {});
const { Text } = Typography;

export default observer((props) => {
  const { KPIStore, DictDataStore, date_picker_store: searchFormStore } = useContext(stores_Context);
  const { sort, initialValue, hides, shows, fieldProps: _fieldProps } = {
    sort: '',
    // initialValue: '',
    initialValue: pick(searchFormStore.formValues, ['DateType', 'year']),
    fieldProps: {},
    hides: [],
    shows: ['DateType', 'years'],
    ...props.searchProps,
  };
  const fieldProps = merge(_fieldProps, { years: { hide_vs: true } });
  const { curObject, objects, KPISubjects, onSearchSubmit } = props;

  return (
    <>
      <Row gutter={16} className="mb-1">
        <Col className="gutter-row mb-n1 p-none" span={24}>
          <SearchForm
            defaultValue={{
              sort, initialValue,
              shows,
              fieldProps,
            }}
            confirmText="查询"
            onSubmit={(_err, obj, form, str) => {
              // console.log('invoke kpi setting search');
              if (typeof onSearchSubmit === 'function') {
                onSearchSubmit(obj, form);
              }
            }}
          />
        </Col>
      </Row>
      <Row gutter={16} className="mb-1 p-s1">
        <Col className="gutter-row mb-n1 p-none" span={24} style={{ backgroundColor: '#ffffff' }}>
          <Tabs
            tabPosition={'left'} defaultActiveKey={KPIStore.settingSubject} activeKey={KPIStore.settingSubject}
            onChange={(sub) => {
              KPIStore.setSettingSubject(sub);
              onSearchSubmit(searchFormStore.formValuesToSub);
            }}
            items={KPISubjects.map((ele, i) => {
              const SubjectTableComponent = subjectComponents[ele.key];
              return {
                ...ele,
                children: <SubjectTableComponent title={ele.label} {...{ curObject, objects, onSearchSubmit }} />
              };
            })}
          />
        </Col>
      </Row>
    </>
  );
});
