import { makeAutoObservable, runInAction, toJS } from 'mobx';
import * as req from '../utils/request';
import moment from 'moment';
import { isEmpty, sortBy, pick, merge, fixTo2Decimals, groupBy, sortKeys, fixToInt, cloneDeep } from '../utils/commons';
import { dataFieldAlias } from './../libs/ht';
import { DATE_FORMAT, SMALL_DATETIME_FORMAT } from './../config';

class Trade {
  constructor(rootStore) {
    this.rootStore = rootStore;
    makeAutoObservable(this);
  }

  /**
   * 年度总额
   */
  async fetchSummaryData(queryData) {
    this.summaryData.loading = true;
    const curQueryData = cloneDeep(queryData);
    curQueryData.groupType = curQueryData?.groupType || 'overview';
    curQueryData.groupDateType = 'year';
    if (isEmpty(curQueryData.DateDiff1)) {
      curQueryData.DateDiff1 = moment(curQueryData.Date1).subtract(1, 'year').format(DATE_FORMAT);
      curQueryData.DateDiff2 = moment(curQueryData.Date2).subtract(1, 'year').format(SMALL_DATETIME_FORMAT);
    }
    const multiData = await this.fetchTradeDataAll((curQueryData));
      const { summary, traditional, biz } = multiData.result1;
      const { summary: summary2, traditional: traditional2, biz: biz2 } = multiData.result2;
      // console.log(JSON.stringify(summary), 'mmmmmmmmmmm', multiData);
        const summaryData = {
          loading: false,
          dataSource: [
            {
              title: '成团',
              col: 6,
              value: summary?.[0]?.ConfirmOrder,
              originVal: summary?.[0]?.ConfirmOrder || 0,
              valueSuffix: undefined,
              // valueSuffix: summary?.[0]?.ConfirmRates ? ` / ${summary?.[0]?.ConfirmRates} %` : undefined,
              // VSrate: summary?.[0]?.ConfirmOrderrate,
              KPIrate: summary?.[0]?.[dataFieldAlias.ConfirmOrder.nestkey.p],
              // hasKPI: !isEmpty(summary?.[0]?.[dataFieldAlias.ConfirmOrder.nestkey.p]),
              hasKPI: false,
              childrenVisible: true,
              kpiVal: summary?.[0]?.[dataFieldAlias.ConfirmOrder.nestkey.v],
              traditional: { title: '传统', value: traditional?.[0]?.ConfirmOrder },
              biz: { title: '商务', value: biz?.[0]?.ConfirmOrder },
              ...(summary2?.[0]
                ? {
                    diff: {
                      label: summary2[0]?.groupDateVal || '对比',
                      value: summary2[0]?.ConfirmOrder || 0,
                      VSrate: summary2[0]?.ConfirmOrder ? fixTo2Decimals(((summary[0]?.ConfirmOrder - summary2[0]?.ConfirmOrder) / summary2[0]?.ConfirmOrder) * 100) : null,
                    },
                  }
                : {}),
            },
            {
              title: '毛利',
              col: 8,
              originVal: summary?.[0]?.SumML || 0,
              value:
                dataFieldAlias.SumML.formatter(summary?.[0]?.SumML || 0) +
                '=' +
                dataFieldAlias.SumML.formatter(traditional?.[0]?.SumML || 0) +
                '+' +
                dataFieldAlias.SumML.formatter(biz?.[0]?.SumML || 0),
              KPIrate: summary?.[0]?.[dataFieldAlias.SumML.nestkey.p],
              hasKPI: false,
              childrenVisible: true,
              kpiVal: summary?.[0]?.[dataFieldAlias.SumML.nestkey.v],
              traditional: { title: '传统', value: traditional?.[0]?.SumML || 0 },
              biz: { title: '商务', value: biz?.[0]?.SumML || 0 },
              ...(summary2?.[0]
                ? {
                    diff: {
                      label: summary2[0]?.groupDateVal || '对比',
                      value: dataFieldAlias.SumML.formatter(summary2[0]?.SumML || 0),
                      VSrate: summary2[0]?.SumML ? fixTo2Decimals(((summary?.[0]?.SumML - summary2[0]?.SumML) / summary2[0]?.SumML) * 100) : null,
                    },
                  }
                : {}),
            },
            {
              title: '完成率',
              col: 5,
              originVal: summary?.[0]?.[dataFieldAlias.SumML.nestkey.p] || 0,
              value: `${summary?.[0]?.[dataFieldAlias.SumML.nestkey.p] || ''}%`,
              hasKPI: false,
              childrenVisible: false,
              kpiVal: 0, // summary?.[0]?.[dataFieldAlias.SumML.nestkey.p],
              traditional: { title: '传统', value: traditional?.[0]?.[dataFieldAlias.SumML.nestkey.p] || 0 },
              biz: { title: '商务', value: biz?.[0]?.[dataFieldAlias.SumML.nestkey.p] || 0 },
              ...(summary2?.[0]
                ? {
                    diff: {
                      label: summary2[0]?.groupDateVal || '对比',
                      value: `${summary2[0]?.[dataFieldAlias.SumML.nestkey.p] || '-'}%`,
                      VSrate: null,
                    },
                  }
                : {}),
            },
            {
              title: '人数',
              col: 5,
              originVal: summary?.[0]?.SumPersonNum || 0,
              value: summary?.[0]?.SumPersonNum,
              // VSrate: summary?.[0]?.SumPersonNumrate,
              // KPIrate: summary?.[0]?.[dataFieldAlias.SumPersonNum.nestkey.p],
              hasKPI: false, // // !isEmpty(summary?.[0]?.[dataFieldAlias.SumPersonNum.nestkey.p]),,
              childrenVisible: true,
              // kpiVal: summary?.[0]?.[dataFieldAlias.SumPersonNum.nestkey.v],
              traditional: { title: '传统', value: traditional?.[0]?.SumPersonNum },
              biz: { title: '商务', value: biz?.[0]?.SumPersonNum },
              ...(summary2?.[0]
                ? {
                  diff: {
                    label: summary2[0]?.groupDateVal || '对比',
                    value: summary2[0]?.SumPersonNum || '',
                    VSrate: summary2[0]?.SumPersonNum ? fixTo2Decimals(((summary?.[0]?.SumPersonNum - summary2[0]?.SumPersonNum) / summary2[0]?.SumPersonNum) * 100) : null,
                  },
                }
              : {}),
            },
          ],
        };
        runInAction(() => {
          this.summaryData = summaryData;
          const kpi = { label: '', value: summary?.[0]?.MLKPIvalue };
          this.summaryData.kpi = kpi;
        });
  }

  /**
   * 时间轴
   */
  async fetchTradeDataByDate(queryData = {}) {
    this.timeData.loading = true;
    queryData = Object.assign({}, this.searchPayloadHome, queryData); // queryData || this.searchPayloadHome;
    queryData.groupType = queryData?.groupType || 'overview';
    Object.assign(queryData, { groupDateType: this.timeLineKey });
    const multiData = await this.fetchTradeDataAll(cloneDeep(queryData));
    const { traditional, biz, summaryRows: summaryRows1, } = multiData.result1;
    // const { summaryRows: summaryRows2, mergeRows: mergeRows2 } = multiData.result2;
    // console.log(biz, 'mmmmmmmm', queryData, multiData);
    const mergeData = [].concat(traditional, biz);
    const dateKeyData = groupBy(mergeData, ele => ele.groupDateVal);
    const sortByDateKey = Object.values(sortKeys(dateKeyData)).reduce( (a, b) => a.concat(b), []);

    runInAction(() => {
      this.timeData.loading = false;
      this.timeData.dataSource = sortByDateKey;
      this.timeData.origin = { summaryRows: summaryRows1 || [] }; // multiData.result1;
    });
  }

  /**
   * 有对比的时间轴
   */
  async fetchTradeDataDiffByDate(queryData = {}) {
    this.timeDiffData.loading = true;
    queryData = Object.assign({}, this.searchPayloadHome, queryData); // queryData || this.searchPayloadHome;
    queryData.groupType = queryData?.groupType || 'overview';
    Object.assign(queryData, { groupDateType: this.timeLineKey });
    const multiData = await this.fetchTradeDataAll(cloneDeep(queryData));
    const { mergeRows: mergeRows1 } = multiData.result1;
    const { mergeRows: mergeRows2 } = multiData.result2;
    // console.log(biz, 'mmmmmmmm', queryData, multiData);

    // 为了图表的X轴一致
    const allDateKey1 = [...new Set(mergeRows1.reduce((rv, vk) => {
      rv.push(vk.groupDateVal);
      return rv;
    }, []))].sort();
    const allDateKey2 = [...new Set(mergeRows2.reduce((rv, vk) => {
      rv.push(vk.groupDateVal);
      return rv;
    }, []))].sort();
    const allLabelDateKeyMapped = {
      ...allDateKey2.reduce((obj, k, i) => ({...obj, [k]: allDateKey1[i] || `_${k}`}), {})
    };
    const mergeKeyDateRows = [].concat(
      mergeRows1 || [],
      (mergeRows2 || []).map((row, ri) => {
        return {
          ...row,
          groupsLabel: `${row.groupsLabel} @${moment(queryData.DateDiff1).year()}`,
          groupDateVal: allLabelDateKeyMapped[row.groupDateVal],
          rawGroupDateVal: row.groupDateVal,
        };
      })
    ).sort(sortBy('groupDateVal'));
    runInAction(() => {
      this.timeDiffData.loading = false;
      this.timeDiffData.dataSource = mergeKeyDateRows;
    });
  }

  /**
   * 事业部年度
   */
  fetchTradeDataByBU(queryData) {
    this.BuData.loading = true;
    Object.assign(queryData, { groupType: 'bu', groupDateType: 'year' });
    this.fetchTradeData(queryData).then((json) => {
      if (json.errcode === 0) {
        const data = json.result1;
        runInAction(() => {
          // 标注KPI

          this.BuData.loading = false;
          this.BuData.dataSource = data;
        });
      }
    });
  }

  /**
   * 成员关系, 年度
   */
  fetchTradeDataByGuestType(queryData) {
    this.BuData.loading = true;
    Object.assign(queryData, { groupType: 'GuestGroupType', groupDateType: 'year' });
    this.fetchTradeData(queryData).then((json) => {
      if (json.errcode === 0) {
        runInAction(() => {
          const data = json.result1;

          this.BuData.loading = false;
          this.BuData.dataSource = data;
        });
      }
    });
  }

  /**
   * 业务区域, 按月
   */
  fetchTradeDataByMonth(queryData) {
    this.sideData.loading = true;
    Object.assign(queryData, { groupType: 'bizarea', groupDateType: 'month' });
    this.fetchTradeData(queryData).then((json) => {
      if (json.errcode === 0) {
        const sortResult = json.result1.sort(sortBy('groupDateVal'));
        const groupsData = sortResult.reduce((r, v) => {
          if (v.groupsLabel ) {
            (r[v.groupsLabel] || (r[v.groupsLabel] = [])).push(v);
          }
          return r;
        }, {});
        const summaryData = Object.keys(groupsData).sort().map(groupsKey => {
          return ['ConfirmOrder', 'SumOrder', 'SumML', 'transactions', 'SumPersonNum'].reduce(
            (r, skey) => ({ ...r, [skey]: groupsData[groupsKey].reduce((a, c) => a + c[skey], 0) }),
            groupsData[groupsKey]?.[0] || {}
          );
        });
        runInAction(() => {
          this.sideData.loading = false;
          this.sideData.dataSource = groupsData;
          this.sideData.monthData = sortResult;
          this.sideData.yearData = summaryData;
        });
      }
    });
  }

  /**
   * TOP
   */
  async fetchTradeDataByType(orderType, queryData) {
    this.topData[orderType] = { loading: true, dataSource: [], originData: [] };
    Object.assign(queryData, { groupType: orderType, groupDateType: 'year' });

    const multiData = await this.fetchTradeDataAll(cloneDeep(queryData));
    const { summary } = multiData.result1;
    // console.log(orderType, summary, 'mmmmmmmm', queryData, multiData);
    runInAction(() => {
      this.topData[orderType].loading = false;
      this.topData[orderType].dataSource = summary;
      this.getTargetsRes(orderType);
    });
  }

  /**
   * 获取业绩数据: 传统订单
   */
  async fetchTradeData(queryData) {
    const json = await req.fetchJSON('/service-Analyse2/GetTradeProcess', queryData);
    // if (json.errcode === 0) {
    //   return json;
    // }
    return json;
  }

  /**
   * 获取业绩数据: 商务订单
   */
  async fetchTradeDataBiz(queryData) {
    const json = await req.fetchJSON('/service-Analyse2/GetTradeProcess_biz', queryData);
    // if (json.errcode === 0) {
    //   return json;
    // }
    return json;
  }

  /**
   * 合并传统和商务
   */
  async fetchTradeDataAll(queryData) {
    const traditional = await this.fetchTradeData(queryData);
    const biz = await this.fetchTradeDataBiz(queryData);
    const rr = parseMergeItem({ traditional, biz });
    return rr;
  }

  /**
   * 从结果中取出目标客户的数据
   */
  getTargetsRes = (orderType) => {
    if ( ! ['GuestGroupType', 'country'].includes(orderType)) {
      return false;
    }
    const countryResMapped = (this.topData[orderType]?.dataSource || []).reduce((a, c) => ({ ...a, [String(c.groupsKey)]: c }), {});
    const guestResMapped = (this.topData[orderType]?.dataSource || []).reduce((a, c) => ({ ...a, [String(c.groupsKey)]: c }), {});
    const targetCountry = merge(this.targetData.targetCountry, pick(countryResMapped, ['3','5','7','18'])); // 美, 加, 英, 澳
    const targetGuest = merge(this.targetData.targetGuest, pick(guestResMapped, ['146001', '146002'])); // 家庭, 夫妻
    this.targetData.targetCountry = targetCountry;
    this.targetData.targetGuest = targetGuest;
    const totalArr = [].concat(Object.values(targetCountry), Object.values(targetGuest));
    const targetTotal = ['ConfirmOrder', 'SumOrder', 'SumML'].reduce((r, skey) => ({...r, [skey]: totalArr.reduce((a, c) => a+(c[skey]), 0)}), {});
    targetTotal.ConfirmRates = fixTo2Decimals((targetTotal.ConfirmOrder/targetTotal.SumOrder)*100);
    targetTotal.groupsLabel = '总数';
    targetTotal.groupsKey = 'targetTotal';
    const targetData = { targetCountry, targetGuest, targetTotal };
    const finalTargetData = merge(this.targetData, targetData);
    this.targetData.targetTotal = targetTotal;
    this.targetTableProps.dataSource = [].concat(Object.values(finalTargetData.targetGuest), Object.values(finalTargetData.targetCountry)); // [finalTargetData.targetTotal], // todo: 总数是重复的
  };

  searchValues = {};
  setSearch(body, form) {
    this.searchPayloadHome = body;
    this.searchValues = form;
  }

  timeLineKey = 'month';
  setTimeLineKey(v) {
    this.timeLineKey = v;
  }

  groupKey = 'overview';
  setGroupKey(v) {
    this.groupKey = v;
  }

  resetData = () => {
    this.summaryData = { loading: false, dataSource: [], kpi: {},  };
    this.timeData = { loading: false, dataSource: [], origin: {}, diff: {} };
    this.timeDiffData = { loading: false, dataSource: [], origin: {}, };
    this.BuData = { loading: false, dataSource: [] };
    this.sideData = { loading: false, dataSource: {}, monthData: [], yearData: [] };
    this.topData = {};
    this.targetData = { targetTotal: {}, targetCountry: {}, targetGuest: {} };
    this.targetTableProps.dataSource = [];
  };

  searchPayloadHome = {};
  summaryData = { loading: false, dataSource: [], kpi: {},  };
  timeData = { loading: false, dataSource: [], origin: {}, diff: {} };
  timeDiffData = { loading: false, dataSource: [], origin: {}, };
  BuData = { loading: false, dataSource: [] };
  sideData = { loading: false, dataSource: {}, monthData: [], yearData: [] };
  topData = {};
  targetData = { targetTotal: {}, targetCountry: {}, targetGuest: {} };
  targetTableProps = { loading: false, columns: [
    {key: 'groupsLabel', title: '', dataIndex: 'groupsLabel', },
    {key: 'SumOrder', title: '订单数', dataIndex: 'SumOrder'},
    {key: 'ConfirmOrder', title: '成行数', dataIndex: 'ConfirmOrder'},
    {key: 'ConfirmRates', title: '成行率', dataIndex: 'ConfirmRates'},
    {key: 'SumML', title: '毛利', dataIndex: 'SumML', render: (v) => dataFieldAlias.SumML.formatter(v)},
    {key: 'transactions', title: '营收', dataIndex: 'transactions', render: (v) => dataFieldAlias.transactions.formatter(v)},
  ], dataSource: [] };
}

export const parseMergeItem = ({traditional, biz}) => {
  return ['result1', 'result2'].reduce((res, resKey) => {
    const mergeItem = {
      traditional: traditional[resKey].map(ele => ({...ele, groupsLabel: `传统 `})), // ${ele.groupsLabel}
      biz: biz[resKey].map(ele => ({...ele, groupsLabel: `商务 `})), // ${ele.groupsLabel}
    };
    const mergeRes = [].concat(traditional[resKey], biz[resKey]);
    const groupsData = mergeRes.reduce((r, v) => {
      if (v.groupsKey ) {
        const groupsKeyLower = v.groupsKey.toLowerCase();
        (r[groupsKeyLower] || (r[groupsKeyLower] = [])).push(v);
      }
      return r;
    }, {});
    const ByDate = sortKeys(groupBy(mergeRes, ele => ele.groupDateVal));
    // 按日期汇总
    const summaryRows = Object.keys(ByDate).map(dateVal => {
      const sumFields = ['ConfirmOrder', 'SumOrder', 'SumML', 'transactions', 'SumPersonNum'].reduce(
        (r, skey) => ({ ...r, [skey]: ByDate[dateVal].reduce((a, c) => a + c[skey], 0) }),
        ByDate[dateVal]?.[0] || {}
      );
      const pickFields = pick(ByDate[dateVal], Object.keys(ByDate[dateVal]).filter(_k => _k.endsWith('KPIvalue') ));
      return {...sumFields, ...pickFields};
    }, {}).map(row => ({...row,
      ConfirmRates: row.SumOrder ? fixTo2Decimals((row.ConfirmOrder/row.SumOrder)*100) : 0,
      MLKPIrates: row.MLKPIvalue ? fixTo2Decimals((row.SumML/row.MLKPIvalue)*100) : 0,
      OrderValue: row.SumOrder ? fixToInt((row.SumML/row.SumOrder)) : 0,
      // groupsLabel: `总 `, // ${row.groupsLabel}
      ConfirmOrderKPIrates: row.ConfirmOrderKPIvalue ? fixTo2Decimals((row.ConfirmOrder/row.ConfirmOrderKPIvalue)*100) : 0,
      OrderKPIrates: row.OrderKPIvalue ? fixTo2Decimals((row.SumOrder/row.OrderKPIvalue)*100) : 0,
      // ConfirmRatesKPIrates: 0, // todo:
    }));
    // 按对象汇总
    const TMapped = traditional[resKey].reduce((r, v) => ({...r, [(v.groupsKey.toLowerCase())]: v}), {});
    const BMapped = biz[resKey].reduce((r, v) => ({...r, [(v.groupsKey.toLowerCase())]: v}), {});
    const summary = Object.keys(groupsData).map(groupsKey => {
      const _groupsKey = groupsKey; // groupsData[groupsKey]?.[0]?.groupsKey || '';
      return ['ConfirmOrder', 'SumOrder', 'SumML', 'transactions', 'SumPersonNum'].reduce(
        (r, skey) => ({ ...r,
          [skey]: groupsData[groupsKey].reduce((a, c) => a + c[skey], 0),
          // [`${skey}_arr`]: groupsData[groupsKey].map(sv => sv[skey]),
          [`${skey}_arr`]: [ _groupsKey ? TMapped?.[_groupsKey]?.[skey] || 0 : 0, _groupsKey ? BMapped?.[_groupsKey]?.[skey] || 0 : 0,],
        }),
        groupsData[groupsKey]?.[0] || {}
      );
    }).map(row => ({...row,
      ConfirmRates: row.SumOrder ? fixTo2Decimals((row.ConfirmOrder/row.SumOrder)*100) : 0,
      MLKPIrates: row.MLKPIvalue ? fixTo2Decimals((row.SumML/row.MLKPIvalue)*100) : 0,
      OrderValue: row.SumOrder ? fixToInt((row.SumML/row.SumOrder)) : 0,
      // groupsLabel: `总 `, // ${row.groupsLabel}
    }));
    // 按每一行
    // const ByDate = sortKeys(groupBy(mergeRes, ele => ele.groupDateVal));
    const allRowsKeysData = groupBy(mergeRes, ele => `${ele.groupsKey.toLowerCase()}@${ele.groupDateVal}`);
    const mergeRows = Object.keys(allRowsKeysData).map(rkey => {
      const _groupsKey = (allRowsKeysData[rkey]?.[0]?.groupsKey || '').toLowerCase();
      const sumFields = ['ConfirmOrder', 'SumOrder', 'SumML', 'transactions', 'SumPersonNum'].reduce(
        (r, skey) => ({ ...r,
          [skey]: allRowsKeysData[rkey].reduce((a, c) => a + c[skey], 0),
          [`${skey}_arr`]: [ _groupsKey ? TMapped?.[_groupsKey]?.[skey] || 0 : 0, _groupsKey ? BMapped?.[_groupsKey]?.[skey] || 0 : 0,],
        }),
        allRowsKeysData[rkey]?.[0] || {}
      );
      const pickFields = pick(allRowsKeysData[rkey], Object.keys(allRowsKeysData[rkey]).filter(_k => _k.endsWith('KPIvalue') ));
      return {...sumFields, ...pickFields};
    }, {}).map(row => ({...row,
      ConfirmRates: row.SumOrder ? fixTo2Decimals((row.ConfirmOrder/row.SumOrder)*100) : 0,
      MLKPIrates: row.MLKPIvalue ? fixTo2Decimals((row.SumML/row.MLKPIvalue)*100) : 0,
      OrderValue: row.SumOrder ? fixToInt((row.SumML/row.SumOrder)) : 0,
      // groupsLabel: `总 `, // ${row.groupsLabel}
      ConfirmOrderKPIrates: row.ConfirmOrderKPIvalue ? fixTo2Decimals((row.ConfirmOrder/row.ConfirmOrderKPIvalue)*100) : 0,
      OrderKPIrates: row.OrderKPIvalue ? fixTo2Decimals((row.SumOrder/row.OrderKPIvalue)*100) : 0,
      // ConfirmRatesKPIrates: 0, // todo:
    }));
    // 合并账户
    const allRowsLabelsData = groupBy(mergeRes, ele => `${ele.groupsLabel.replace(/\([^)]*\)/gi, '').toLowerCase()}@${ele.groupDateVal}`);
    const mergeLabelsRows = Object.keys(allRowsLabelsData).map(rkey => {
      const _groupsLabel = rkey.split('@')[0].toLowerCase();
      const _groupsKey = (allRowsLabelsData[rkey]?.[0]?.groupsKey || '').toLowerCase();
      const sumFields = [
        'ConfirmOrder', 'SumOrder', 'SumML', 'transactions', 'SumPersonNum',
        'ConfirmOrderKPIvalue', 'OrderKPIvalue', 'MLKPIvalue',
    ].reduce(
        (r, skey) => ({ ...r,
          [skey]: allRowsLabelsData[rkey].reduce((a, c) => a + c[skey], 0),
          // [`${skey}_arr`]: [ _groupsKey ? TMapped?.[_groupsKey]?.[skey] || 0 : 0, _groupsKey ? BMapped?.[_groupsKey]?.[skey] || 0 : 0,],
        }),
        allRowsLabelsData[rkey]?.[0] || {}
      );
      const pickFields = pick(allRowsLabelsData[rkey], Object.keys(allRowsLabelsData[rkey]).filter(_k => _k.endsWith('KPIvalue') ));
      return {...pickFields, ...sumFields, groupsLabel: _groupsLabel, groupsKey: _groupsLabel };
    }, {}).map(row => ({...row,
      ConfirmRates: row.SumOrder ? fixTo2Decimals((row.ConfirmOrder/row.SumOrder)*100) : 0,
      MLKPIrates: row.MLKPIvalue ? fixTo2Decimals((row.SumML/row.MLKPIvalue)*100) : 0,
      OrderValue: row.SumOrder ? fixToInt((row.SumML/row.SumOrder)) : 0,
      ConfirmOrderKPIrates: row.ConfirmOrderKPIvalue ? fixTo2Decimals((row.ConfirmOrder/row.ConfirmOrderKPIvalue)*100) : 0,
      OrderKPIrates: row.OrderKPIvalue ? fixTo2Decimals((row.SumOrder/row.OrderKPIvalue)*100) : 0,
      // ConfirmRatesKPIrates: 0, // todo:
    }));
    return Object.assign(res, { [resKey]: Object.assign({}, mergeItem, { mergeRows, summary, summaryRows, mergeLabelsRows }) });
  }, {});
};

export default Trade;
