import React, { useContext } from 'react';
import { observer } from 'mobx-react';
import { stores_Context } from '../config';
import { Row, Col, Table, Space, Typography, Divider } from 'antd';
import SearchForm from './../components/search/SearchForm';
import { VSTag, TableExportBtn } from './../components/Data';
import { CruiseAgency } from './../libs/ht';

const { Text } = Typography;

export default observer((props) => {
  const { date_picker_store: searchFormStore } = useContext(stores_Context);
  const { HotelCruiseStore, date_picker_store } = useContext(stores_Context);
  const { loading, dataSource, summaryRow } = HotelCruiseStore.cruise;

  const { formValues, siderBroken } = searchFormStore;

  const tableSorter = (a, b, colName) => a[colName] - b[colName];
  const tableExportDataRow = (col1, col2) => [col1, col2].filter((r) => r).join(' VS ');
  const tableProps = {
    size: 'small',
    bordered: true,
    pagination: false,
    columns: [
      { title: '产品',
        sorter: (a, b) => a.ProductName.localeCompare(b.ProductName, 'zh-CN'),children: [{ title: summaryRow.ProductName, dataIndex: 'ProductName', key: 'ProductName' }] },
      {
        title: '房间数',
        sorter: (a, b) => tableSorter(a, b, 'TotalNum'),
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.TotalNum}
                    {summaryRow.TotalNumPercent ? <Text type="secondary"> VS {summaryRow.CPTotalNum}</Text> : null}
                  </Text>
                  {summaryRow.TotalNumPercent && <VSTag diffPercent={summaryRow.TotalNumPercent} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.TotalNum, summaryRow.CPTotalNum),
            dataExport: (v, r) => tableExportDataRow(r.TotalNum, r.CPTotalNum),
            dataIndex: 'TotalNum',
            key: 'TotalNum',
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.CPTotalNum ? <Text type="secondary"> VS {r.CPTotalNum}</Text> : null}
                  </Text>
                  {r.CPTotalNum && <VSTag diffPercent={r.TotalNumPercent} />}
                </Space>
              </>
            ),
          },
        ],
      },
      {
        title: '人数',
        sorter: (a, b) => tableSorter(a, b, 'TotalPersonNum'),
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.TotalPersonNum}
                    {summaryRow.TotalPersonNumPercent ? <Text type="secondary"> VS {summaryRow.CPTotalPersonNum}</Text> : null}
                  </Text>
                  {summaryRow.TotalPersonNumPercent && <VSTag diffPercent={summaryRow.TotalPersonNumPercent} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.TotalPersonNum, summaryRow.CPTotalPersonNum),
            dataExport: (v, r) => tableExportDataRow(r.TotalPersonNum, r.CPTotalPersonNum),
            dataIndex: 'TotalPersonNum',
            key: 'TotalPersonNum',
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.CPTotalPersonNum ? <Text type="secondary"> VS {r.CPTotalPersonNum}</Text> : null}
                  </Text>
                  {r.CPTotalNum && <VSTag diffPercent={r.TotalPersonNumPercent} />}
                </Space>
              </>
            ),
          },
        ],
      },
      {
        title: '总利润',
        sorter: (a, b) => tableSorter(a, b, 'TotalProfit'),
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.TotalProfit}
                    {summaryRow.TotalProfitPercent ? <Text type="secondary"> VS {summaryRow.CPTotalProfit}</Text> : null}
                  </Text>
                  {summaryRow.TotalProfitPercent && <VSTag diffPercent={summaryRow.TotalProfitPercent} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.TotalProfit, summaryRow.CPTotalProfit),
            dataExport: (v, r) => tableExportDataRow(r.TotalProfit, r.CPTotalProfit),
            dataIndex: 'TotalProfit',
            key: 'TotalProfit',
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.CPTotalProfit ? <Text type="secondary"> VS {r.CPTotalProfit}</Text> : null}
                  </Text>
                  {r.CPTotalNum && <VSTag diffPercent={r.TotalProfitPercent} />}
                </Space>
              </>
            ),
          },
        ],
      },
      // { title: '单订船', dataIndex: '', key: '' },
      // { title: '订单含行程', dataIndex: '', key: '' },
      // { title: '国籍', dataIndex: '', key: '' },
    ],
  };

  return (
    <>
      <Row gutter={16} className={siderBroken ? '' : 'sticky-top'}>
        <Col md={24} lg={24} xxl={24}>
          <SearchForm
            defaultValue={{
              initialValue: {
                ...date_picker_store.formValues,
                ...HotelCruiseStore.searchValues,
              },
              // 'countryArea',
              shows: [
                'DepartmentList',
                'orderStatus',
                'dates',
                'keyword',
                'cruiseDirection',
                'agency',
                'cruiseBookType',
                'country', // 'roomsRange', 'personRange'
              ],
              sort: { keyword: 101, agency: 110, cruiseDirection: 102, country: 104 },
              fieldProps: {
                keyword: { placeholder: '产品名', col: 4 },
                DepartmentList: { show_all: true, mode: 'multiple' },
                orderStatus: { show_all: true },
                cruiseDirection: { show_all: true },
                agency: { defaultOptions: CruiseAgency, autoGet: false },
                // years: { hide_vs: false },
              },
            }}
            onSubmit={(_err, obj, form) => {
              HotelCruiseStore.setSearchValues(obj, form);
              HotelCruiseStore.getCruiseData(obj);
            }}
          />
        </Col>
      </Row>
      <section>
        <Divider orientation="right" >
          <TableExportBtn label={'游船'} {...{ columns: tableProps.columns, dataSource }} />
        </Divider>
        <Table {...tableProps} {...{ loading, dataSource }} rowKey={(record) => record.ProductName} />
      </section>
    </>
  );
});
