import { useContext, useEffect } from 'react';
import { Row, Col, Space, Table, List, Typography, Divider } from 'antd';
import { stores_Context } from '../config';
import { observer } from 'mobx-react';
import { NavLink, useParams } from 'react-router-dom';
import 'moment/locale/zh-cn';
import SearchForm from './../components/search/SearchForm';
import { TableExportBtn } from './../components/Data';

const DestinationGroupList = () => {
  const { destinationId } = useParams();
  const { customerServicesStore, date_picker_store } = useContext(stores_Context);

  useEffect(() => {
    customerServicesStore.fetchDistGroupInfoByCountry(destinationId);
    customerServicesStore.fetchGroupListByDestinationId(destinationId);
  }, []);

  const destinationGroupList = customerServicesStore.destinationGroupList;
  const destinationGroupListColumns = customerServicesStore.destinationGroupListColumns;
  const nationality_count_data = customerServicesStore.nationality_count_data;
  const { inProgress } = customerServicesStore;

  return (
    <>
      <Space direction="vertical" style={{ width: '100%' }}>
        <Row gutter={{ md: 24 }} justify="end">
          <Col span={24}>
            <NavLink to={'/destination/group/count'}>返回</NavLink>
          </Col>
          <Col className="gutter-row" span={24}>
            <SearchForm
              defaultValue={{
                initialValue: {
                  ...date_picker_store.formValues,
                  countryArea: { key: 'china', label: '国内' },
                  ...customerServicesStore.searchValues,
                },
                shows: ['departureDateType', 'DepartmentList', 'dates', 'orderStatus'],
                fieldProps: {
                  DepartmentList: { show_all: true },
                  orderStatus: { show_all: true },
                  countryArea: { show_all: false },
                  dates: { hide_vs: true },
                  departureDateType: { disabledKeys: ['applyDate'] },
                },
              }}
              onSubmit={(_err, obj, form) => {
                customerServicesStore.setSearchValues(obj, form);
                customerServicesStore.fetchGroupListByDestinationId(destinationId);
                customerServicesStore.fetchDistGroupInfoByCountry(destinationId);
                customerServicesStore.fetchDestinationGroupCount();
              }}
            />
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <Typography.Title level={3}>国籍统计</Typography.Title>
            <Divider orientation="right" plain>
              <TableExportBtn label={'国籍统计'} {...{ columns: nationality_count_data.destinationGroupByCountryListColumns, dataSource: nationality_count_data.destinationGroupByCountryList }} />
            </Divider>
            <Table
              sticky
              id="destinationGroupByCountry"
              dataSource={nationality_count_data.destinationGroupByCountryList}
              columns={nationality_count_data.destinationGroupByCountryListColumns}
              size="small"
              rowKey={(record) => record.key}
              loading={nationality_count_data.loading}
              pagination={false}
              scroll={{ x: 1000 }}
            />
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <Table
              sticky
              dataSource={destinationGroupList}
              columns={destinationGroupListColumns}
              size="small"
              rowKey={(record) => record.key}
              loading={inProgress}
              pagination={false}
              scroll={{ x: 600 }}
              expandable={{
                expandedRowRender: (record) => (
                  <List itemLayout="horizontal">
                    <List.Item>
                      <List.Item.Meta title="经过城市" description={record.PassCity} />
                    </List.Item>
                    <List.Item>
                      <List.Item.Meta title="地接社" description={record.VendorList} />
                    </List.Item>
                  </List>
                ),
              }}
            />
          </Col>
        </Row>
      </Space>
    </>
  );
};

export default observer(DestinationGroupList);
