import React, { useContext } from 'react';
import { observer } from 'mobx-react';
import { stores_Context } from '../config';
import { Row, Col, Table, Space, Typography, Divider } from 'antd';
import SearchForm from './../components/search/SearchForm';
import { VSTag, TableExportBtn } from './../components/Data';
const { Text } = Typography;

export default observer((props) => {
  const { date_picker_store: searchFormStore } = useContext(stores_Context);
  const { date_picker_store, HotelCruiseStore } = useContext(stores_Context);
  const { loading, dataSource, summaryRow } = HotelCruiseStore.hotel;

  const { formValues, siderBroken } = searchFormStore;

  const tableSorter = (a, b, colName) => a[colName] - b[colName];
  const tableExportDataRow = (col1, col2) => [col1, col2].filter((r) => r).join(' VS ');

  const tableProps = {
    size: 'small',
    bordered: true,
    pagination: false,
    columns: [
      {
        title: '目的地',
        sorter: (a, b) => a.CityName.localeCompare(b.CityName, 'zh-CN'),
        children: [{ title: summaryRow.CityName, dataIndex: 'CityName', key: 'CityName' }],
      },
      {
        title: '总间夜',
        sorter: (a, b) => tableSorter(a, b, 'TotalNum'),
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.TotalNum}
                    {summaryRow.TotalNumPercent ? <Text type="secondary"> VS {summaryRow.CPTotalNum}</Text> : null}
                  </Text>
                  {summaryRow.TotalNumPercent && <VSTag diffPercent={summaryRow.TotalNumPercent} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.TotalNum, summaryRow.CPTotalNum),
            dataIndex: 'TotalNum',
            key: 'TotalNum',
            dataExport: (v, r) => tableExportDataRow(r.TotalNum, r.CPTotalNum),
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.CPTotalNum ? <Text type="secondary"> VS {r.CPTotalNum}</Text> : null}
                  </Text>
                  {r.CPTotalNum && <VSTag diffPercent={r.TotalNumPercent} />}
                </Space>
              </>
            ),
          },
        ],
      },
      {
        title: '主推',
        sorter: (a, b) => tableSorter(a, b, 'RecomendNum'),
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.RecomendNum}
                    {summaryRow.RecomendNumPercent ? <Text type="secondary"> VS {summaryRow.CPRecomendNum}</Text> : null}
                  </Text>
                  {summaryRow.RecomendNumPercent && <VSTag diffPercent={summaryRow.RecomendNumPercent} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.RecomendNum, summaryRow.CPRecomendNum),
            dataIndex: 'RecomendNum',
            key: 'RecomendNum',
            dataExport: (v, r) => tableExportDataRow(r.RecomendNum, r.CPRecomendNum),
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.CPRecomendNum ? <Text type="secondary"> VS {r.CPRecomendNum}</Text> : null}
                  </Text>
                  {r.CPRecomendNum && <VSTag diffPercent={r.RecomendNumPercent} />}
                </Space>
              </>
            ),
          },
        ],
      },
      {
        title: '使用比例',
        children: [
          {
            title: (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {summaryRow.RecommendRate_100}
                    {summaryRow.RecommendRateDelta ? <Text type="secondary"> VS {summaryRow.CPRecommendRate_100}</Text> : null}
                  </Text>
                  {summaryRow.RecommendRateDelta && <VSTag diffPercent={summaryRow.RecommendRateDelta} />}
                </Space>
              </>
            ),
            titleX: tableExportDataRow(summaryRow.RecommendRate_100, summaryRow.CPRecommendRate_100),
            dataIndex: 'RecommendRate_100',
            key: 'RecommendRate_100',
            dataExport: (v, r) => tableExportDataRow(r.RecommendRate_100, r.CPRecommendRate_100),
            render: (v, r) => (
              <>
                <Space direction={'vertical'}>
                  <Text strong>
                    {v}
                    {r.RecommendRateDelta !== undefined && <Text type="secondary"> VS {r.CPRecommendRate_100}</Text>}
                  </Text>
                  {r.RecommendRateDelta !== undefined && <VSTag diffPercent={r.RecommendRateDelta} />}
                </Space>
              </>
            ),
          },
        ],
      },
    ],
  };

  return (
    <>
      <Row gutter={16} className={siderBroken ? '' : 'sticky-top'}>
        <Col md={24} lg={24} xxl={24}>
          <SearchForm
            defaultValue={{
              initialValue: {
                ...date_picker_store.formValues,
                ...HotelCruiseStore.searchValues,
              },
              // 'countryArea', 'DateType', 'dates', 'hotelRecommandRate',
              shows: ['DepartmentList', 'countryArea', 'orderStatus', 'hotelBookType', 'hotelStar', 'DateType', 'dates'],
              sort: { DateType: 101, dates: 102 },
              fieldProps: {
                DepartmentList: { show_all: true, mode: 'multiple' },
                countryArea: { show_all: true },
                orderStatus: { show_all: true },
                hotelBookType: { show_all: true },
                hotelRecommandRate: { show_all: true },
                // years: { hide_vs: false },
                DateType: { disabledKeys: ['applyDate'] },
              },
            }}
            onSubmit={(_err, obj, form) => {
              HotelCruiseStore.setSearchValues(obj, form);
              HotelCruiseStore.getHotelData(obj);
            }}
          />
        </Col>
      </Row>
      <section>
        <Divider orientation="right" >
          <TableExportBtn label={'酒店'} {...{ columns: tableProps.columns, dataSource }} />
        </Divider>
        <Table {...tableProps} bordered {...{ loading, dataSource }} rowKey={(record) => record.CityName} />
      </section>
    </>
  );
});
