import React, { useContext } from 'react';
import { Link } from 'react-router-dom';
import { observer } from 'mobx-react';
import { stores_Context } from '../../config';
import { InfoCircleOutlined } from '@ant-design/icons';
import { Row, Col, Table, Tooltip } from 'antd';
import SearchForm from '../../components/search/SearchForm';
import { pick } from '../../utils/commons';

const COLOR_SETS = [
  "#FFFFFF",
  // "#5B8FF9",
  // "#FF6B3B",
  "#9FB40F",
  "#76523B",
  "#DAD5B5",
  "#E19348",
  "#F383A2",
];

const transparentHex = '1A';

export default observer((props) => {
  const { SalesCRMDataStore, date_picker_store: searchFormStore } = useContext(stores_Context);

  // const { formValues, siderBroken } = searchFormStore;
  const { formValues, formValuesToSub, siderBroken } = searchFormStore;
  const _formValuesToSub = pick(formValuesToSub, ['DepartmentList', 'WebCode']);
  const { resetData, process } = SalesCRMDataStore;
  const operatorObjects = process.details.map((v) => ({ key: v.groupsKey, value: v.groupsKey, label: v.groupsLabel, text: v.groupsLabel }));

  const pageRefresh = async (obj) => {
    resetData('process');
    Promise.allSettled([
      SalesCRMDataStore.getProcessData({
        ...(obj || _formValuesToSub),
        // ...subParam,
        // groupType: 'overview',
        groupType: 'dept',
        groupDateType: '',
      }),
      SalesCRMDataStore.getProcessData({
        ...(obj || _formValuesToSub),
        // ...subParam,
        groupType: 'operator',
        groupDateType: '',
      }),
    ]);
  };

  const tableSorter = (a, b, colName) => (a.groupType !== 'operator' ? -1 : b.groupType !== 'operator' ? 0 : a[colName] - b[colName]);
  const percentageRender = val => val ? `${val}%` : '';

  const activityTableProps = {
    rowKey: 'groupsKey',
    pagination: { pageSize: 10, showSizeChanger: false },
    size: 'small',
    rowClassName: (record, rowIndex) => {
      return record.groupType === 'operator' ? '': 'ant-tag-blue';
    },
    showSorterTooltip: false,
    columns: [
      { title: '', dataIndex: 'groupsLabel', key: 'groupsLabel', width: '6rem',
        filterSearch: true,
        filters: operatorObjects.sort((a, b) => a.text.localeCompare(b.text)),
        onFilter: (value, record) => record.groupsKey === value || record.groupType !== 'operator',
        render: (text, record) => (record.groupType !== 'operator' ? text : <Link to={`/op_risk/sales/${record.groupsKey}`}>{text}</Link>),
      }, // 维度: 顾问
      {
        title: '顾问动作',
        key: 'date',
        children: [
          { title: () => (
            <>
              首次响应率24H{' '}
              <Tooltip title="达到24H内回复的订单数/总订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'firstTouch24', key: 'firstTouch24', render: percentageRender,
            sorter: (a, b) => tableSorter(a, b, 'firstTouch24'),
          },
          { title: () => (
            <>
              48H内报价率{' '}
              <Tooltip title="达到48H内报价的订单数/总订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'firstQuote48', key: 'firstQuote48',render: percentageRender ,
            sorter: (a, b) => tableSorter(a, b, 'firstQuote48'), },
          { title: () => (
            <>
              一次报价率{' '}
              <Tooltip title="首次报价的订单数/总订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'quote1', key: 'quote1',render: percentageRender ,
            sorter: (a, b) => tableSorter(a, b, 'quote1'), },
          { title: () => (
            <>
              二次报价率{' '}
              <Tooltip title="二次报价的订单数/一次报价后回复数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'quote2', key: 'quote2',render: percentageRender ,
            sorter: (a, b) => tableSorter(a, b, 'quote2'), },
          { title: () => (
            <>
              &gt;50条会话{' '}
              <Tooltip title=">50条会话的订单数/总订单">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'turnsGT50', key: 'turnsGT50', render: percentageRender ,
            sorter: (a, b) => tableSorter(a, b, 'turnsGT50'), },
          { title: () => (
            <>
              违规数{' '}
              <Tooltip title="未遵循24H回复的订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'violations', key: 'violations',
            sorter: (a, b) => tableSorter(a, b, 'violations'), },
        ],
      },
      {
        title: '客人回复',
        key: 'department',
        children: [
          {
            title: () => (
            <>
              首次回复率24H{' '}
              <Tooltip title="达到24H内回复的订单数/总订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'firstReply24', key: 'firstReply24',
            render: (_, r) => ({
              props: { style: { backgroundColor: COLOR_SETS[1]+transparentHex } },
              children: percentageRender(_),
            }),
            sorter: (a, b) => tableSorter(a, b, 'firstReply24'), },
          {
            title: () => (
            <>
              48H内报价回复率{' '}
              <Tooltip title="48H内报价后的回复数/报价的订单数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'replyQuote48', key: 'replyQuote48',
            render: (_, r) => ({
              props: { style: { backgroundColor: COLOR_SETS[1]+transparentHex } },
              children: percentageRender(_),
            }),
            sorter: (a, b) => tableSorter(a, b, 'replyQuote48'), },
          {
            title: () => (
            <>
              一次报价回复率{' '}
              <Tooltip title="一次报价后回复率/订单总数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'replyQuote1', key: 'replyQuote1',
            render: (_, r) => ({
              props: { style: { backgroundColor: COLOR_SETS[1]+transparentHex } },
              children: percentageRender(_),
            }),
            sorter: (a, b) => tableSorter(a, b, 'replyQuote1'), },
          {
            title: () => (
            <>
              二次报价回复率{' '}
              <Tooltip title="二次报价后回复数/一次报价后回复数">
                <InfoCircleOutlined />
              </Tooltip>
            </>
          ), dataIndex: 'replyQuote2', key: 'replyQuote2',
            render: (_, r) => ({
              props: { style: { backgroundColor: COLOR_SETS[1]+transparentHex } },
              children: percentageRender(_),
            }),
            sorter: (a, b) => tableSorter(a, b, 'replyQuote2'), },
        ],
      },
    ],
  };

  const riskTableProps = {
    rowKey: 'groupsKey',
    pagination: { pageSize: 10, showSizeChanger: false },
    size: 'small',
    rowClassName: (record, rowIndex) => {
      return record.groupType === 'operator' ? '': 'ant-tag-blue';
    },
    showSorterTooltip: false,
    columns: [
      { title: '', dataIndex: 'groupsLabel', key: 'groupsLabel', width: '6rem',
        filterSearch: true,
        filters: operatorObjects.sort((a, b) => a.text.localeCompare(b.text)),
        onFilter: (value, record) => record.groupsKey === value || record.groupType !== 'operator',
        render: (text, record) => (record.groupType !== 'operator' ? text : <Link to={`/op_risk/sales/${record.groupsKey}`}>{text}</Link>),
      }, // 维度: 顾问
      { title: '>24H回复', dataIndex: 'lostTouch24', key: 'lostTouch24',
        sorter: (a, b) => tableSorter(a, b, 'lostTouch24'),
       },
      { title: '首次报价周期>48h', dataIndex: 'lostQuote48', key: 'lostQuote48',
        sorter: (a, b) => tableSorter(a, b, 'lostQuote48'),
       },
      { title: '报价次数<1次', dataIndex: 'lostQuote1', key: 'lostQuote1',
        sorter: (a, b) => tableSorter(a, b, 'lostQuote1'),
       },
      { title: '报价次数<2次', dataIndex: 'lostQuote2', key: 'lostQuote2',
        sorter: (a, b) => tableSorter(a, b, 'lostQuote2'),
       },
    ],
  };

  return (
    <>
      <Row gutter={16} className={siderBroken ? '' : 'sticky-top'}>
        <Col md={24} lg={24} xxl={24}>
          <SearchForm
            defaultValue={{
              initialValue: {
                ...formValues,
                ...SalesCRMDataStore.searchValues,
              },
              shows: ['DepartmentList', 'WebCode', 'DateType', 'dates'],
              fieldProps: {
                DepartmentList: { show_all: false, mode: 'multiple', col: 5 },
                WebCode: { show_all: false, mode: 'multiple', col: 5 },
                dates: { hide_vs: true },
              },
            }}
            onSubmit={(_err, obj, form, str) => {
              SalesCRMDataStore.setSearchValues(obj, form);
              pageRefresh(obj);
            }}
          />
        </Col>
      </Row>
      <section>
        <Table {...activityTableProps} bordered dataSource={[...process.dataSource, ...process.details]} loading={process.loading} sticky />
      </section>
      <section>
        <h3>未成行订单 数量</h3>
        <Table {...riskTableProps} bordered dataSource={[...process.dataSource, ...process.details]} loading={process.loading} sticky />
      </section>
    </>
  );
});
